package bcn.generator;

import bcn.lang.BehaviorDef;
import bcn.lang.Connection;
import bcn.lang.Controller;
import bcn.lang.Declaration;
import bcn.lang.InfluenceDef;
import bcn.lang.InputPortDef;
import bcn.lang.Instance;
import bcn.lang.JavaParam;
import bcn.lang.PortDef;
import bcn.lang.PortRef;
import com.google.common.base.Objects;
import com.google.common.collect.Iterators;
import java.util.Iterator;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.IteratorExtensions;

@SuppressWarnings("all")
public class LangGenerator implements IGenerator {
  public void doGenerate(final Resource resource, final IFileSystemAccess fsa) {
    this.generate(resource, fsa);
  }
  
  public void generate(final Resource resource, final IFileSystemAccess fsa) {
    TreeIterator<EObject> _allContents = resource.getAllContents();
    Iterator<Controller> _filter = Iterators.<Controller>filter(_allContents, Controller.class);
    Controller _head = IteratorExtensions.<Controller>head(_filter);
    this.generate(_head, fsa);
  }
  
  public void generate(final Controller controller, final IFileSystemAccess fsa) {
    EList<Declaration> _decls = controller.getDecls();
    for (final Declaration m : _decls) {
      if ((m instanceof BehaviorDef)) {
        String _pckg = controller.getPckg();
        this.generate(((BehaviorDef) m), fsa, _pckg);
      }
    }
    String _pckg_1 = controller.getPckg();
    String _plus = (_pckg_1 + "_stub/");
    String _name = controller.getName();
    String _plus_1 = (_plus + _name);
    String _plus_2 = (_plus_1 + "_stub.java");
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _pckg_2 = controller.getPckg();
    _builder.append(_pckg_2, "");
    _builder.append("_stub;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import com.cyberbotics.webots.controller.Robot;");
    _builder.newLine();
    _builder.append("import java.util.Random;");
    _builder.newLine();
    _builder.append("import bcn.Scheduler;");
    _builder.newLine();
    _builder.append("import bcn.coord.*;");
    _builder.newLine();
    _builder.append("import ");
    String _pckg_3 = controller.getPckg();
    _builder.append(_pckg_3, "");
    _builder.append(".*;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.newLine();
    _builder.append("public abstract class ");
    String _name_1 = controller.getName();
    _builder.append(_name_1, "");
    _builder.append("_stub {");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("protected Scheduler scheduler;");
    _builder.newLine();
    {
      EList<Declaration> _decls_1 = controller.getDecls();
      for(final Declaration i : _decls_1) {
        {
          if ((i instanceof Instance)) {
            _builder.append("    ");
            _builder.append("protected ");
            String _behavior = ((Instance) i).getBehavior();
            _builder.append(_behavior, "    ");
            String _types = ((Instance) i).getTypes();
            _builder.append(_types, "    ");
            _builder.append(" ");
            String _name_2 = ((Instance) i).getName();
            _builder.append(_name_2, "    ");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("public abstract void setupAndRun();");
    _builder.newLine();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("protected void connectAndRun(Robot robot) {");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("connectAndRun(robot, 32);");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("protected void connectAndRun(Robot robot, int timeStep) {");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("connectAndRun(robot, timeStep, new Random(0));");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("protected void connectAndRun(Robot robot, int timeStep, Random rnd) {");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("// connect behaviors");
    _builder.newLine();
    {
      EList<Declaration> _decls_2 = controller.getDecls();
      for(final Declaration c : _decls_2) {
        {
          if ((c instanceof Connection)) {
            _builder.append("        ");
            PortRef _src = ((Connection) c).getSrc();
            String _portRef = this.portRef(_src);
            _builder.append(_portRef, "        ");
            _builder.append(".connectTo(");
            EList<PortRef> _dests = ((Connection) c).getDests();
            String _listPortRefs = this.listPortRefs(_dests);
            _builder.append(_listPortRefs, "        ");
            _builder.append(");");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder.append("        ");
    _builder.append("// run scheduler");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("scheduler = new Scheduler(robot, timeStep, rnd");
    {
      EList<Declaration> _decls_3 = controller.getDecls();
      for(final Declaration i_1 : _decls_3) {
        {
          if ((i_1 instanceof Instance)) {
            _builder.append(", ");
            String _name_3 = ((Instance) i_1).getName();
            _builder.append(_name_3, "        ");
          }
        }
      }
    }
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("        ");
    _builder.append("scheduler.run();");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    fsa.generateFile(_plus_2, _builder);
    String _pckg_4 = controller.getPckg();
    String _plus_3 = (_pckg_4 + "/");
    String _name_4 = controller.getName();
    String _plus_4 = (_plus_3 + _name_4);
    String _plus_5 = (_plus_4 + ".java");
    StringConcatenation _builder_1 = new StringConcatenation();
    _builder_1.append("package ");
    String _pckg_5 = controller.getPckg();
    _builder_1.append(_pckg_5, "");
    _builder_1.append(";");
    _builder_1.newLineIfNotEmpty();
    _builder_1.newLine();
    _builder_1.newLine();
    _builder_1.append("public class ");
    String _name_5 = controller.getName();
    _builder_1.append(_name_5, "");
    _builder_1.append(" extends ");
    String _pckg_6 = controller.getPckg();
    _builder_1.append(_pckg_6, "");
    _builder_1.append("_stub.");
    String _name_6 = controller.getName();
    _builder_1.append(_name_6, "");
    _builder_1.append("_stub {");
    _builder_1.newLineIfNotEmpty();
    _builder_1.append("    ");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.append("static public void main(String[] args) {");
    _builder_1.newLine();
    _builder_1.append("        ");
    String _name_7 = controller.getName();
    _builder_1.append(_name_7, "        ");
    _builder_1.append(" controller = new ");
    String _name_8 = controller.getName();
    _builder_1.append(_name_8, "        ");
    _builder_1.append("();");
    _builder_1.newLineIfNotEmpty();
    _builder_1.append("        ");
    _builder_1.append("controller.setupAndRun();");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.append("}");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.append("@Override");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.append("public void setupAndRun() {");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.append("com.cyberbotics.webots.controller.Robot robot;");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.append("// init sensors");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.append("//TODO");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.append("// init actuators");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.append("//TODO");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.append("// setup behaviors");
    _builder_1.newLine();
    {
      EList<Declaration> _decls_4 = controller.getDecls();
      for(final Declaration d : _decls_4) {
        {
          if ((d instanceof Instance)) {
            _builder_1.append("        ");
            String _name_9 = ((Instance) d).getName();
            _builder_1.append(_name_9, "        ");
            _builder_1.append(" = new ");
            String _behavior_1 = ((Instance) d).getBehavior();
            _builder_1.append(_behavior_1, "        ");
            _builder_1.append("(...);");
            _builder_1.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder_1.append("        ");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.append("// run controller");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.append("connectAndRun(robot);");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.append("}");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.newLine();
    _builder_1.append("}");
    _builder_1.newLine();
    fsa.generateFile(_plus_5, _builder_1);
  }
  
  private String portRef(final PortRef ref) {
    StringConcatenation _builder = new StringConcatenation();
    Instance _behavior = ref.getBehavior();
    String _name = _behavior.getName();
    _builder.append(_name, "");
    _builder.append(".get_");
    String _port = ref.getPort();
    _builder.append(_port, "");
    _builder.append("(");
    {
      String _index = ref.getIndex();
      boolean _notEquals = (!Objects.equal(_index, null));
      if (_notEquals) {
        String _index_1 = ref.getIndex();
        _builder.append(_index_1, "");
      }
    }
    _builder.append(")");
    return _builder.toString();
  }
  
  private String listPortRefs(final EList<PortRef> refs) {
    final Iterator<PortRef> i = refs.iterator();
    PortRef p = i.next();
    StringBuilder _stringBuilder = new StringBuilder();
    final StringBuilder s = _stringBuilder;
    String _portRef = this.portRef(p);
    s.append(_portRef);
    boolean _hasNext = i.hasNext();
    boolean _while = _hasNext;
    while (_while) {
      {
        PortRef _next = i.next();
        p = _next;
        s.append(", ");
        String _portRef_1 = this.portRef(p);
        s.append(_portRef_1);
      }
      boolean _hasNext_1 = i.hasNext();
      _while = _hasNext_1;
    }
    return s.toString();
  }
  
  public void generate(final BehaviorDef behavior, final IFileSystemAccess fsa, final String pckg) {
    String _plus = (pckg + "_stub/");
    String _name = behavior.getName();
    String _plus_1 = (_plus + _name);
    String _plus_2 = (_plus_1 + "_stub.java");
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(pckg, "");
    _builder.append("_stub;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import bcn.*;");
    _builder.newLine();
    _builder.append("import ");
    _builder.append(pckg, "");
    _builder.append(".*;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.newLine();
    _builder.append("public abstract class ");
    String _name_1 = behavior.getName();
    _builder.append(_name_1, "");
    _builder.append("_stub");
    String _types = behavior.getTypes();
    _builder.append(_types, "");
    _builder.append(" extends Behavior {");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    {
      EList<InputPortDef> _inputs = behavior.getInputs();
      for(final InputPortDef in : _inputs) {
        {
          PortDef _port = in.getPort();
          boolean _notEquals = (!Objects.equal(_port, null));
          if (_notEquals) {
            _builder.append("    ");
            _builder.append("protected final InputPort<");
            PortDef _port_1 = in.getPort();
            String _type = _port_1.getType();
            _builder.append(_type, "    ");
            _builder.append(">");
            {
              PortDef _port_2 = in.getPort();
              String _array = _port_2.getArray();
              boolean _notEquals_1 = (!Objects.equal(_array, null));
              if (_notEquals_1) {
                _builder.append("[]");
              }
            }
            _builder.append(" _");
            PortDef _port_3 = in.getPort();
            String _name_2 = _port_3.getName();
            _builder.append(_name_2, "    ");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
          } else {
            InfluenceDef _influence = in.getInfluence();
            boolean _notEquals_2 = (!Objects.equal(_influence, null));
            if (_notEquals_2) {
              _builder.append("    ");
              _builder.append("protected final InputInfluence");
              {
                InfluenceDef _influence_1 = in.getInfluence();
                String _array_1 = _influence_1.getArray();
                boolean _notEquals_3 = (!Objects.equal(_array_1, null));
                if (_notEquals_3) {
                  _builder.append("[]");
                }
              }
              _builder.append(" _");
              InfluenceDef _influence_2 = in.getInfluence();
              String _name_3 = _influence_2.getName();
              _builder.append(_name_3, "    ");
              _builder.append(";");
              _builder.newLineIfNotEmpty();
            }
          }
        }
      }
    }
    {
      EList<PortDef> _outputs = behavior.getOutputs();
      for(final PortDef out : _outputs) {
        _builder.append("    ");
        _builder.append("protected final OutputPort<");
        String _type_1 = out.getType();
        _builder.append(_type_1, "    ");
        _builder.append(">");
        {
          String _array_2 = out.getArray();
          boolean _notEquals_4 = (!Objects.equal(_array_2, null));
          if (_notEquals_4) {
            _builder.append("[]");
          }
        }
        _builder.append(" _");
        String _name_4 = out.getName();
        _builder.append(_name_4, "    ");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("public ");
    String _name_5 = behavior.getName();
    _builder.append(_name_5, "    ");
    _builder.append("_stub(");
    EList<JavaParam> _params = behavior.getParams();
    String _formalParams = this.formalParams(_params);
    _builder.append(_formalParams, "    ");
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    {
      EList<InputPortDef> _inputs_1 = behavior.getInputs();
      for(final InputPortDef in_1 : _inputs_1) {
        {
          PortDef _port_4 = in_1.getPort();
          boolean _notEquals_5 = (!Objects.equal(_port_4, null));
          if (_notEquals_5) {
            {
              PortDef _port_5 = in_1.getPort();
              String _array_3 = _port_5.getArray();
              boolean _equals = Objects.equal(_array_3, null);
              if (_equals) {
                _builder.append("        ");
                _builder.append("_");
                PortDef _port_6 = in_1.getPort();
                String _name_6 = _port_6.getName();
                _builder.append(_name_6, "        ");
                _builder.append(" = new ");
                {
                  String _buffer = in_1.getBuffer();
                  boolean _equals_1 = Objects.equal(_buffer, null);
                  if (_equals_1) {
                    _builder.append("InputRegister<");
                    PortDef _port_7 = in_1.getPort();
                    String _type_2 = _port_7.getType();
                    _builder.append(_type_2, "        ");
                    _builder.append(">()");
                  } else {
                    _builder.append("InputBuffer<");
                    PortDef _port_8 = in_1.getPort();
                    String _type_3 = _port_8.getType();
                    _builder.append(_type_3, "        ");
                    _builder.append(">(");
                    String _buffer_1 = in_1.getBuffer();
                    _builder.append(_buffer_1, "        ");
                    _builder.append(")");
                  }
                }
                _builder.append(";");
                _builder.newLineIfNotEmpty();
              } else {
                _builder.append("        ");
                _builder.append("_");
                PortDef _port_9 = in_1.getPort();
                String _name_7 = _port_9.getName();
                _builder.append(_name_7, "        ");
                _builder.append(" = new ");
                {
                  String _buffer_2 = in_1.getBuffer();
                  boolean _equals_2 = Objects.equal(_buffer_2, null);
                  if (_equals_2) {
                    _builder.append("InputRegister");
                  } else {
                    _builder.append("InputBuffer");
                  }
                }
                _builder.append("[");
                PortDef _port_10 = in_1.getPort();
                String _array_4 = _port_10.getArray();
                _builder.append(_array_4, "        ");
                _builder.append("];");
                _builder.newLineIfNotEmpty();
                _builder.append("        ");
                _builder.append("for (int i=0; i<");
                PortDef _port_11 = in_1.getPort();
                String _array_5 = _port_11.getArray();
                _builder.append(_array_5, "        ");
                _builder.append("; i++) {");
                _builder.newLineIfNotEmpty();
                _builder.append("        ");
                _builder.append("    ");
                _builder.append("_");
                PortDef _port_12 = in_1.getPort();
                String _name_8 = _port_12.getName();
                _builder.append(_name_8, "            ");
                _builder.append("[i] = new ");
                {
                  String _buffer_3 = in_1.getBuffer();
                  boolean _equals_3 = Objects.equal(_buffer_3, null);
                  if (_equals_3) {
                    _builder.append("InputRegister<");
                    PortDef _port_13 = in_1.getPort();
                    String _type_4 = _port_13.getType();
                    _builder.append(_type_4, "            ");
                    _builder.append(">()");
                  } else {
                    _builder.append("InputBuffer<");
                    PortDef _port_14 = in_1.getPort();
                    String _type_5 = _port_14.getType();
                    _builder.append(_type_5, "            ");
                    _builder.append(">(");
                    String _buffer_4 = in_1.getBuffer();
                    _builder.append(_buffer_4, "            ");
                    _builder.append(")");
                  }
                }
                _builder.append(";");
                _builder.newLineIfNotEmpty();
                _builder.append("        ");
                _builder.append("}");
                _builder.newLine();
              }
            }
          } else {
            InfluenceDef _influence_3 = in_1.getInfluence();
            boolean _notEquals_6 = (!Objects.equal(_influence_3, null));
            if (_notEquals_6) {
              {
                InfluenceDef _influence_4 = in_1.getInfluence();
                String _array_6 = _influence_4.getArray();
                boolean _equals_4 = Objects.equal(_array_6, null);
                if (_equals_4) {
                  _builder.append("        ");
                  _builder.append("_");
                  InfluenceDef _influence_5 = in_1.getInfluence();
                  String _name_9 = _influence_5.getName();
                  _builder.append(_name_9, "        ");
                  _builder.append(" = new InputInfluence((float)(");
                  InfluenceDef _influence_6 = in_1.getInfluence();
                  String _min = _influence_6.getMin();
                  _builder.append(_min, "        ");
                  _builder.append("), (float)(");
                  InfluenceDef _influence_7 = in_1.getInfluence();
                  String _max = _influence_7.getMax();
                  _builder.append(_max, "        ");
                  _builder.append("), (float)(");
                  InfluenceDef _influence_8 = in_1.getInfluence();
                  String _dec = _influence_8.getDec();
                  _builder.append(_dec, "        ");
                  _builder.append("));");
                  _builder.newLineIfNotEmpty();
                } else {
                  _builder.append("        ");
                  _builder.append("_");
                  InfluenceDef _influence_9 = in_1.getInfluence();
                  String _name_10 = _influence_9.getName();
                  _builder.append(_name_10, "        ");
                  _builder.append(" = new InputInfluence[");
                  InfluenceDef _influence_10 = in_1.getInfluence();
                  String _array_7 = _influence_10.getArray();
                  _builder.append(_array_7, "        ");
                  _builder.append("];");
                  _builder.newLineIfNotEmpty();
                  _builder.append("        ");
                  _builder.append("for (int i=0; i<");
                  InfluenceDef _influence_11 = in_1.getInfluence();
                  String _array_8 = _influence_11.getArray();
                  _builder.append(_array_8, "        ");
                  _builder.append("; i++) {");
                  _builder.newLineIfNotEmpty();
                  _builder.append("        ");
                  _builder.append("    ");
                  _builder.append("_");
                  InfluenceDef _influence_12 = in_1.getInfluence();
                  String _name_11 = _influence_12.getName();
                  _builder.append(_name_11, "            ");
                  _builder.append("[i] = new InputInfluence((float)(");
                  InfluenceDef _influence_13 = in_1.getInfluence();
                  String _min_1 = _influence_13.getMin();
                  _builder.append(_min_1, "            ");
                  _builder.append("), (float)(");
                  InfluenceDef _influence_14 = in_1.getInfluence();
                  String _max_1 = _influence_14.getMax();
                  _builder.append(_max_1, "            ");
                  _builder.append("), (float)(");
                  InfluenceDef _influence_15 = in_1.getInfluence();
                  String _dec_1 = _influence_15.getDec();
                  _builder.append(_dec_1, "            ");
                  _builder.append("));");
                  _builder.newLineIfNotEmpty();
                  _builder.append("        ");
                  _builder.append("}");
                  _builder.newLine();
                }
              }
            }
          }
        }
      }
    }
    {
      EList<PortDef> _outputs_1 = behavior.getOutputs();
      for(final PortDef out_1 : _outputs_1) {
        {
          String _array_9 = out_1.getArray();
          boolean _equals_5 = Objects.equal(_array_9, null);
          if (_equals_5) {
            _builder.append("        ");
            _builder.append("_");
            String _name_12 = out_1.getName();
            _builder.append(_name_12, "        ");
            _builder.append(" = new OutputPort<");
            String _type_6 = out_1.getType();
            _builder.append(_type_6, "        ");
            _builder.append(">();");
            _builder.newLineIfNotEmpty();
          } else {
            _builder.append("        ");
            _builder.append("_");
            String _name_13 = out_1.getName();
            _builder.append(_name_13, "        ");
            _builder.append(" = new OutputPort[");
            String _array_10 = out_1.getArray();
            _builder.append(_array_10, "        ");
            _builder.append("];");
            _builder.newLineIfNotEmpty();
            _builder.append("        ");
            _builder.append("for (int i=0; i<");
            String _array_11 = out_1.getArray();
            _builder.append(_array_11, "        ");
            _builder.append("; i++) {");
            _builder.newLineIfNotEmpty();
            _builder.append("        ");
            _builder.append("    ");
            _builder.append("_");
            String _name_14 = out_1.getName();
            _builder.append(_name_14, "            ");
            _builder.append("[i] = new OutputPort<");
            String _type_7 = out_1.getType();
            _builder.append(_type_7, "            ");
            _builder.append(">();");
            _builder.newLineIfNotEmpty();
            _builder.append("        ");
            _builder.append("}");
            _builder.newLine();
          }
        }
      }
    }
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.append("    ");
    _builder.newLine();
    {
      EList<InputPortDef> _inputs_2 = behavior.getInputs();
      for(final InputPortDef in_2 : _inputs_2) {
        {
          PortDef _port_15 = in_2.getPort();
          boolean _notEquals_7 = (!Objects.equal(_port_15, null));
          if (_notEquals_7) {
            _builder.append("    ");
            _builder.append("public final InputPort<");
            PortDef _port_16 = in_2.getPort();
            String _type_8 = _port_16.getType();
            _builder.append(_type_8, "    ");
            _builder.append("> get_");
            PortDef _port_17 = in_2.getPort();
            String _name_15 = _port_17.getName();
            _builder.append(_name_15, "    ");
            _builder.append("(");
            {
              PortDef _port_18 = in_2.getPort();
              String _array_12 = _port_18.getArray();
              boolean _notEquals_8 = (!Objects.equal(_array_12, null));
              if (_notEquals_8) {
                _builder.append("int id");
              }
            }
            _builder.append(") {");
            _builder.newLineIfNotEmpty();
            _builder.append("    ");
            _builder.append("    ");
            _builder.append("return _");
            PortDef _port_19 = in_2.getPort();
            String _name_16 = _port_19.getName();
            _builder.append(_name_16, "        ");
            {
              PortDef _port_20 = in_2.getPort();
              String _array_13 = _port_20.getArray();
              boolean _notEquals_9 = (!Objects.equal(_array_13, null));
              if (_notEquals_9) {
                _builder.append("[id]");
              }
            }
            _builder.append(";");
            _builder.newLineIfNotEmpty();
            _builder.append("    ");
            _builder.append("}");
            _builder.newLine();
            _builder.append("    ");
            _builder.newLine();
          } else {
            InfluenceDef _influence_16 = in_2.getInfluence();
            boolean _notEquals_10 = (!Objects.equal(_influence_16, null));
            if (_notEquals_10) {
              _builder.append("    ");
              _builder.append("public final InputInfluence get_");
              InfluenceDef _influence_17 = in_2.getInfluence();
              String _name_17 = _influence_17.getName();
              _builder.append(_name_17, "    ");
              _builder.append("(");
              {
                InfluenceDef _influence_18 = in_2.getInfluence();
                String _array_14 = _influence_18.getArray();
                boolean _notEquals_11 = (!Objects.equal(_array_14, null));
                if (_notEquals_11) {
                  _builder.append("int id");
                }
              }
              _builder.append(") {");
              _builder.newLineIfNotEmpty();
              _builder.append("    ");
              _builder.append("    ");
              _builder.append("return _");
              InfluenceDef _influence_19 = in_2.getInfluence();
              String _name_18 = _influence_19.getName();
              _builder.append(_name_18, "        ");
              {
                InfluenceDef _influence_20 = in_2.getInfluence();
                String _array_15 = _influence_20.getArray();
                boolean _notEquals_12 = (!Objects.equal(_array_15, null));
                if (_notEquals_12) {
                  _builder.append("[id]");
                }
              }
              _builder.append(";");
              _builder.newLineIfNotEmpty();
              _builder.append("    ");
              _builder.append("}");
              _builder.newLine();
              _builder.append("    ");
              _builder.newLine();
            }
          }
        }
      }
    }
    {
      EList<PortDef> _outputs_2 = behavior.getOutputs();
      for(final PortDef out_2 : _outputs_2) {
        _builder.append("    ");
        _builder.append("public final OutputPort<");
        String _type_9 = out_2.getType();
        _builder.append(_type_9, "    ");
        _builder.append("> get_");
        String _name_19 = out_2.getName();
        _builder.append(_name_19, "    ");
        _builder.append("(");
        {
          String _array_16 = out_2.getArray();
          boolean _notEquals_13 = (!Objects.equal(_array_16, null));
          if (_notEquals_13) {
            _builder.append("int id");
          }
        }
        _builder.append(") {");
        _builder.newLineIfNotEmpty();
        _builder.append("    ");
        _builder.append("    ");
        _builder.append("return _");
        String _name_20 = out_2.getName();
        _builder.append(_name_20, "        ");
        {
          String _array_17 = out_2.getArray();
          boolean _notEquals_14 = (!Objects.equal(_array_17, null));
          if (_notEquals_14) {
            _builder.append("[id]");
          }
        }
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("    ");
        _builder.append("}");
        _builder.newLine();
        _builder.append("    ");
        _builder.newLine();
      }
    }
    _builder.append("}");
    _builder.newLine();
    fsa.generateFile(_plus_2, _builder);
    String _plus_3 = (pckg + "/");
    String _name_21 = behavior.getName();
    String _plus_4 = (_plus_3 + _name_21);
    String _plus_5 = (_plus_4 + ".java");
    StringConcatenation _builder_1 = new StringConcatenation();
    _builder_1.append("package ");
    _builder_1.append(pckg, "");
    _builder_1.append(";");
    _builder_1.newLineIfNotEmpty();
    _builder_1.newLine();
    _builder_1.newLine();
    _builder_1.append("public class ");
    String _name_22 = behavior.getName();
    _builder_1.append(_name_22, "");
    String _types_1 = behavior.getTypes();
    _builder_1.append(_types_1, "");
    _builder_1.append(" extends ");
    _builder_1.append(pckg, "");
    _builder_1.append("_stub.");
    String _name_23 = behavior.getName();
    _builder_1.append(_name_23, "");
    _builder_1.append("_stub");
    String _types_2 = behavior.getTypes();
    _builder_1.append(_types_2, "");
    _builder_1.append(" {");
    _builder_1.newLineIfNotEmpty();
    _builder_1.append("    ");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.append("public ");
    String _name_24 = behavior.getName();
    _builder_1.append(_name_24, "    ");
    _builder_1.append("(");
    EList<JavaParam> _params_1 = behavior.getParams();
    String _formalParams_1 = this.formalParams(_params_1);
    _builder_1.append(_formalParams_1, "    ");
    _builder_1.append(") {");
    _builder_1.newLineIfNotEmpty();
    _builder_1.append("        ");
    _builder_1.append("super(");
    EList<JavaParam> _params_2 = behavior.getParams();
    String _actualParams = this.actualParams(_params_2);
    _builder_1.append(_actualParams, "        ");
    _builder_1.append(");");
    _builder_1.newLineIfNotEmpty();
    _builder_1.append("    ");
    _builder_1.append("}");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.append("@Override");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.append("public void step() {");
    _builder_1.newLine();
    _builder_1.append("        ");
    _builder_1.append("//TODO");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.append("}");
    _builder_1.newLine();
    _builder_1.append("    ");
    _builder_1.newLine();
    _builder_1.append("}");
    _builder_1.newLine();
    fsa.generateFile(_plus_5, _builder_1);
  }
  
  private String formalParams(final EList<JavaParam> params) {
    return this.listParams(params, true);
  }
  
  private String actualParams(final EList<JavaParam> params) {
    return this.listParams(params, false);
  }
  
  private String listParams(final EList<JavaParam> params, final boolean formal) {
    int _size = params.size();
    boolean _lessThan = (_size < 1);
    if (_lessThan) {
      return "";
    } else {
      final Iterator<JavaParam> i = params.iterator();
      JavaParam p = i.next();
      StringBuilder _stringBuilder = new StringBuilder();
      final StringBuilder s = _stringBuilder;
      if (formal) {
        String _type = p.getType();
        String _plus = (_type + " ");
        s.append(_plus);
      }
      String _name = p.getName();
      s.append(_name);
      boolean _hasNext = i.hasNext();
      boolean _while = _hasNext;
      while (_while) {
        {
          JavaParam _next = i.next();
          p = _next;
          s.append(", ");
          if (formal) {
            String _type_1 = p.getType();
            String _plus_1 = (_type_1 + " ");
            s.append(_plus_1);
          }
          String _name_1 = p.getName();
          s.append(_name_1);
        }
        boolean _hasNext_1 = i.hasNext();
        _while = _hasNext_1;
      }
      return s.toString();
    }
  }
}
