package bcn;

import java.util.ArrayList;
import java.util.Arrays;


public class OutputPort<T> {
    
    private ArrayList<InputRef<T>> recipients;
    
    
    public OutputPort() {
        this.recipients = new ArrayList<InputRef<T>>();
    }
    
    
    public OutputPort(InputRef<T>... recipients) {
        this.recipients = new ArrayList<InputRef<T>>(recipients.length);
        this.recipients.addAll(Arrays.asList(recipients));
    }
    
    
    public synchronized void connectTo(InputRef<T>... recipients) {
        this.recipients.addAll(Arrays.asList(recipients));
    }
    
    
    public synchronized void connectTo(InputPort<T>... recipients) {
        LocalInputRef<T>[] refs = new LocalInputRef[recipients.length];
        for (int i=0; i<recipients.length; i++) {
            refs[i] = new LocalInputRef<T>(recipients[i]);
        }
        this.recipients.addAll(Arrays.asList(refs));
    }
    
    
    public synchronized void disconnectFrom(InputRef<T> recipient) {
        this.recipients.remove(recipient);
    }
    
    
    public synchronized void send(T msg) {
        for (InputRef<T> r: recipients) {
            r.send(msg);
        }
    }
    
}
