package bcn.coord;

import bcn.InputPort;
import bcn.InputRegister;
import bcn.Behavior;
import bcn.OutputPort;


public abstract class FusionCoord<T> extends Behavior {
    
    protected final InputPort<MsgAct<T>>[] _in;
    protected final OutputPort<MsgAct<T>> _out;
    protected final OutputPort<T> _outMsg;
    protected final OutputPort<Float> _outAct;
    
    
    public FusionCoord(int count) {
    	if (count < 2) {
    		throw new IllegalArgumentException("A coordinator with less than 2 inputs doesn't make sense!");
    	}
        _in = new InputPort[count];
        for (int i=0; i<count; i++) {
            _in[i] = new InputRegister<MsgAct<T>>();
            _in[i].put(MsgAct.ACT_0);  // fill inputs with dummy data
        }
        _out = new OutputPort<MsgAct<T>>();
        _outMsg = new OutputPort<T>();
        _outAct = new OutputPort<Float>();
    }
    
    
    public final InputPort<MsgAct<T>> get_in(int id) {
        return _in[id];
    }
    
    
    public final OutputPort<MsgAct<T>> get_out() {
        return _out;
    }
    
    
    public final OutputPort<T> get_outMsg() {
        return _outMsg;
    }
    
    
    public final OutputPort<Float> get_outAct() {
        return _outAct;
    }
    
    
    @Override
    public void step() {
    	MsgAct<T> winner = fuse();  // call user specific code
        // send output activation level
        _out.send(winner);
        _outAct.send(winner.act);
        if (winner.act > 0) {
            _outMsg.send(winner.msg);  // plain messages mustn't be sent if activation is 0
        }
    }
    
    
    /** User specific method to fuse the inputs:
      * the user must ignore inputs with activation equal to 0;
      * if no useful output can be computed it should return MsgAct.ACT_0. */
    protected abstract MsgAct<T> fuse();
    
}
