package foraging;

import bcn.coord.PriorityCoord;
import com.cyberbotics.webots.controller.Camera;
import com.cyberbotics.webots.controller.Compass;
import com.cyberbotics.webots.controller.DifferentialWheels;
import com.cyberbotics.webots.controller.DistanceSensor;
import com.cyberbotics.webots.controller.GPS;
import com.cyberbotics.webots.controller.Servo;


public class CtrlWanderRechargeForage extends foraging_stub.CtrlWanderRechargeForage_stub {
	
	static public void main(String[] args) {
		CtrlWanderRechargeForage controller = new CtrlWanderRechargeForage();
    	controller.setupAndRun();
    }
	
	
    @Override
	public void setupAndRun() {
        DifferentialWheels robot = new DifferentialWheels();
        
        // init sensors
        DistanceSensor[] ds = new DistanceSensor[8];
        for (int i=0; i<ds.length; i++) {
            ds[i] = robot.getDistanceSensor("ds" + i);
            ds[i].enable(Misc.CONTROL_STEP);
        }
        robot.batterySensorEnable(Misc.CONTROL_STEP);
        GPS gps = robot.getGPS("gps");
        gps.enable(Misc.CONTROL_STEP*2);
        Compass compass = robot.getCompass("compass");
        compass.enable(Misc.CONTROL_STEP*2);
        Camera cam = robot.getCamera("camera");
        cam.enable(Misc.CONTROL_STEP*2);
        
        // init actuators
        Servo liftGrip = robot.getServo("servo");
        Servo leftGrip = robot.getServo("left grip");
        Servo rightGrip = robot.getServo("right grip");
        liftGrip.enablePosition(Misc.CONTROL_STEP);
        leftGrip.enablePosition(Misc.CONTROL_STEP);
        leftGrip.enableMotorForceFeedback(Misc.CONTROL_STEP);
        liftGrip.setVelocity(2.0f);
        leftGrip.setVelocity(0.02f);
        rightGrip.setVelocity(0.02f);
        
        // setup modules
        detect = new Detect(cam);
        gripper = new Gripper(liftGrip, leftGrip, rightGrip);
        forage = new Forage(ds, gps);
        avoid = new Avoid(ds);
        recharge = new Recharge(robot, gps);
        coordAttract = new PriorityCoord<Location>(2);
        attract = new Attract(gps, compass);
        repulse = new Repulse(gps, compass);
        wander = new Wander(4);
        coordDrive = new PriorityCoord<Drive.Speed>(5);
        drive = new Drive(robot);
        
        WinnerInspector winDrive = new WinnerInspector("Drive", "forage", "avoid", "attract", "repulse", "wander");
        coordDrive.get_winner().connectTo(winDrive.getInputPort());
        WinnerInspector winAttract = new WinnerInspector("Attract", "recharge", "forage");
        coordAttract.get_winner().connectTo(winAttract.getInputPort());
        
        // run controller
        connectAndRun(robot, Misc.CONTROL_STEP, Misc.RANDOM);
        
        winDrive.dispose();
        winAttract.dispose();
    }
    
}
